#include <string.h>
#include "arg.h"
#include "log.h"
#include "dep.h"
#include "db.h"
#include "generator.h"

/* Runs when a custom command line argument is found
 returns true if no furhter argument processing should be done */
int hook_custom_arg(const char *key, const char *value)
{
	if (strcmp(key, "numeric") == 0) {
		if ((strcmp(value, "int") == 0) || (strcmp(value, "double") == 0))
			put("/local/numeric", value);
		else {
			fprintf(stderr, "Error: invalid numeric format. Must be int or double\n");
			exit(1);
		}
		return 1;
	}
	return 0;
}


/* Runs before anything else */
int hook_preinit()
{
	return 0;
}

/* Runs after initialization */
int hook_postinit()
{
	db_mkdir("/local");

	report("Configuring libmawk.\n");
	logprintf(0, "Configuring libmawk.\n");
	return 0;
}

/* Runs after all arguments are read and parsed */
int hook_postarg()
{
	if (get("/local/numeric") == NULL)
		put("/local/numeric", "double");
	return 0;
}

/* Runs when things should be detected for the host system */
int hook_detect_host()
{
	return 0;
}

/* Runs when things should be detected for the target system */
int hook_detect_target()
{
	/* if there was no custom requirement from the command line, run all requirements in non-fatal mode */
	if (num_custom_reqs < 1) {
		require("cc/cc", 0, 1);
		require("cc/fpic", 0, 1);
		require("fstools/chmodx", 0, 1);
		require("fstools/cp", 0, 1);
		require("fstools/rm", 0, 1);
		require("fstools/ln", 0, 1);
		require("fstools/mkdir", 0, 1);
		require("sys/types/size_t/includes", 0, 0);
		require("libs/fs/realpath/presents", 0, 0);
		require("libs/env/putenv", 0, 1);
		require("libs/io/pipe/presents", 0, 0);
		require("libs/math/cc/log/*", 0, 1);
		require("libs/snprintf", 0, 1);
		require("libs/math/nan/*", 0, 0);
		require("libs/math/isnan/*", 0, 0);
		require("libs/math/nanop/*", 0, 0);

		require("parsgen/bison", 0, 0);
		printf("Numeric format: %s\n", get("/local/numeric"));
	}
	return 0;
}

#ifdef GENCALL
/* If enabled, generator implements ###call *### and generator_callback is
   the callback function that will be executed upon ###call### */
void generator_callback(char *cmd, char *args)
{
	printf("* generator_callback: '%s' '%s'\n", cmd, args);
}
#endif

/* Runs after detection hooks, should generate the output (Makefiles, etc.) */
int hook_generate()
{
	printf("Generating libmawk/Makefile... ");
	fflush(stdout);
	if (generate("../src/libmawk/Makefile.conf.in", "../src/libmawk/Makefile.conf") == 0)
		printf("OK\n");
	else
		printf("failed\n");

	printf("Generating libmawk/conf.h... ");
	fflush(stdout);
	if (generate("../src/libmawk/conf.h.in", "../src/libmawk/conf.h") == 0)
		printf("OK\n");
	else
		printf("failed\n");

	printf("Generating awklib/Makefile... ");
	fflush(stdout);
	if (generate("../src/awklib/Makefile.in", "../src/awklib/Makefile") == 0)
		printf("OK\n");
	else
		printf("failed\n");

	printf("Generating awklib/regression/Makefile... ");
	fflush(stdout);
	db_mkdir("/local");
	if (generate("../src/awklib/regression/Makefile.in", "../src/awklib/regression/Makefile") == 0)
		printf("OK\n");
	else
		printf("failed\n");

	return 0;
}

/* Runs before everything is uninitialized */
void hook_preuninit()
{
}

/* Runs at the very end, when everything is already uninitialized */
void hook_postuninit()
{
}

